//+------------------------------------------------------------------+
//|                                                  Correlation.mq4 |
//|                                Copyright  2008, Victor Nicolaev |
//|                                             http://vinin.ucoz.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Victor Nicolaev"
#property link      "http://vinin.ucoz.ru"

#property indicator_separate_window
#property indicator_buffers   3
#property indicator_color1 Green
#property indicator_color2 Yellow
#property indicator_color3 Red
#property indicator_maximum   1
#property indicator_minimum  -1
#property indicator_level1    0.7
#property indicator_level2    0.0
#property indicator_level3   -0.7


//---- input parameters

extern string    _Symbol="";
extern int       nPeriod0= 30;
extern int       nPeriod1=120;
extern int       nPeriod2=480;
extern int       nShift0= 30;
extern int       nShift1=120;
extern int       nShift2=480;
extern int        Limit=1440;

//---- buffers
double Buffer0[];
double Buffer1[];
double Buffer2[];

bool  bSymbol;
double X0[],Y0[];
double X1[],Y1[];
double X2[],Y2[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
   for (int i=0;i<3;i++) SetIndexStyle(i,DRAW_LINE);

   SetIndexBuffer(0,Buffer0);
   SetIndexBuffer(1,Buffer1);
   SetIndexBuffer(2,Buffer2);
   
   bSymbol=false;
   if (_Symbol=="") {
      bSymbol=true;
      _Symbol=Symbol();
   }

   string indicator_name="AutoCorrelation()";   
   if (!bSymbol) indicator_name="Correlation("+Symbol()+"/"+_Symbol+")";   

   IndicatorShortName(indicator_name);
   
   SetIndexDrawBegin(0,nPeriod0+nShift0);
   SetIndexDrawBegin(1,nPeriod1+nShift1);
   SetIndexDrawBegin(2,nPeriod2+nShift2);
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()  {
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
   int i,limit,counted_bars=IndicatorCounted();
   
   if ( counted_bars < 0 ) return(-1);

   limit=Bars;
   if(counted_bars>0) limit=Bars-counted_bars;

   limit--;
   if (limit>Limit && Limit>0) limit=Limit;
   for( i=limit; i>=0; i--) {
      Buffer0[i]=correlation(i, nPeriod0, nShift0);
      Buffer1[i]=correlation(i, nPeriod1, nShift1);
      Buffer2[i]=correlation(i, nPeriod2, nShift2);
   }
   return(0);
}

double correlation(int pos, int CorrBar, int ShiftBar=0){
   double RetVal=0;
   double sx=0, sy=0, dx=0, dy=0,dxy=0,sdxy=0;
   int ix,iy;   
   sx = iMA(Symbol(), 0, CorrBar, 0, MODE_SMA, PRICE_CLOSE, pos);
   sy = iMA(_Symbol , 0, CorrBar, 0, MODE_SMA, PRICE_CLOSE, pos+ShiftBar);

   
   for (int i=0; i<CorrBar; i++)  {
      dx+=MathPow(iClose(Symbol(),0,pos+i)-sx,2);
      iy=pos+i+ShiftBar;
      if (!bSymbol) {
         ix=iTime(Symbol(),0,pos+i+ShiftBar);
         iy=iBarShift(_Symbol, 0, ix);
         if (iTime(_Symbol, 0, iy)>ix) iy++;
      }
      dy+=MathPow(iClose(_Symbol, 0,iy)-sy,2);
      dxy+=(iClose(Symbol(),0,pos+i)-sx)*(iClose(_Symbol,0,iy)-sy);
   }
   
   sdxy=MathSqrt(dx*dy);

   if (sdxy>0) RetVal=dxy/sdxy;
   return(RetVal);
}


